/* **********************************************************************
 * ztokenizer is really just "getWords". 
 * It takes a pointer to a file, some parameters, and calls gotWord every
 * time it finds a word in that file. Returns 0 when done.
 * ********************************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "zwordcount.h"
#include "ztokenizer.h"
#include "zsearcher.h"



//FIXME use isalpha
int IsAkeeper (
   char* instr,
   int opt_digits
) ;

void lc (char* a_word ) ;
void finalcheck (char* a_word) ;




/* ************************************************************************
 * getWords does a char-by-char read of the file so that I can control
 * what I call a "word", particularly regarding handling hyphens, digits,
 * and later on plurals and the use of Unicode instead of ASCII.
 *
 * That is, instead of arguing about what a word *is*, doing a char-by-char
 * read means that the program is easy to modify.
 * If "Alas" is different from "Alas!", or "bob" is different from "Bob",
 * or "OhMyGod" is different from "Oh-my-god", I have left the code open
 * for easy changing. If you want digits in your words, this 1 d0n't care.
 * ************************************************************************ */
int getWords (
  FILE* file_ptr,
  int opt_split,
  int opt_digits,
  int opt_all
  )
{
  int    at_eof ;
  char   charIN ;
  char   instr[2] ;
  char   theword[128] ;
  int    thewordlength ;
  int    hyphenfound ;
  int    wordcount ;

  at_eof        = 0 ;
  wordcount     = 0 ;
  hyphenfound   = 0 ;
  thewordlength = 0 ;
  instr[1]      = '\0' ;

  while (at_eof == 0)
  {
    charIN = fgetc(file_ptr) ;
    instr[0] = charIN ;

    // ==================================================
    if (charIN == EOF) {
      at_eof = 1 ;
    }

    // ==================================================
    else if (hyphenfound == 1)     // hyphen at eoline, part II, figure out how to handle it.
    {
      if (strcmp("\n",instr) == 0) // Likely hyphenated because line is too long.
      {
        hyphenfound = 0 ;          // Don't save this hyphen, just go to the next character and re-join the word.
      }
      else if (strcmp("-",instr) == 0) // A second hyphen, skip this second hyphen too, and
      {                                // call that which has been gathered so far a word.
        hyphenfound = 0 ; 
        theword[thewordlength] = '\0' ;
        ++wordcount ;
        if (thewordlength > 0)
        {
          finalcheck(theword) ;
        }
        thewordlength = 0 ;          
      }
      else // A single hyphenated word. But do not preserve the hyphen if "split" is set.
      {
        if (opt_split == 1)
        {
          theword[thewordlength] = '\0' ;
          wordcount++ ;
          if (thewordlength > 0)
          {
            finalcheck(theword) ;
          }
          thewordlength = 0 ;          
          if (IsAkeeper(instr,opt_digits)) 
          {
            hyphenfound = 0 ;
            theword[thewordlength++] = charIN ;
          }
        } /* if (opt_split == 1) */
        else
        {
          /* add the hyphen back in*/
          hyphenfound = 0 ;
          theword[thewordlength++] = '-' ;
          theword[thewordlength++] = charIN ;
        }
      }
    } /* end of if (hyphenfound == 1) */

    // ==================================================
    else if (strcmp("-",instr) == 0)
    {
      hyphenfound = 1 ; // Note that a hyphen was found but do not include it in theword.
    }


    // ==================================================
    else if ((IsAkeeper(instr,opt_digits) || (strcmp("'",instr) == 0)) )
    {
      hyphenfound = 0 ;
      theword[thewordlength++] = charIN ; 
      if (thewordlength > 127)
      {
        theword[127] = '\0' ;
        fprintf(stderr, "\nBIG word. Truncating to : %s\n",theword) ;
        wordcount++ ; // A word found, give it away
        finalcheck(theword) ;
        thewordlength = 0 ;
      }
    }

    // ==================================================
    /* end of word. Save what has accumulated so far. */
    else
    {
      if (hyphenfound == 0)
      {
        if (thewordlength >= 1)
        {
          theword[thewordlength] = '\0' ;
          wordcount++ ; 
          if (thewordlength > 0) 
          {  
            finalcheck(theword) ;
          }
        }
        thewordlength = 0 ;
      }
   }
 } /* End of while (at_eof == 0) */

  return (0) ;
}

/* ***********************************************************************
 * We've got rules.
 * Anything that is only a ' is not a word.
 * A ' starting a word should be removed
 * A ' ending   a word should be removed
 * Apologies to Huck Finn.
 * ********************************************************************* */
void finalcheck(char* theword) 
{
  lc(theword) ;

  int i ;
  if (theword[0] == '\'')
  {
    i = 0 ;
    while (theword[i] != '\0') 
    { 
     theword[i] = theword[i+1] ;
     i++ ;
    }
    theword[i] = '\0' ;
  }
  if (theword[strlen(theword)-1] == '\'')
  {
    theword[strlen(theword)-1] = '\0' ;
  }

  if ((strcmp("'",theword) != 0) && (strlen(theword)>0))
  {
    gotWord(theword) ;
  }

}


/* ***********************************************************************
 * It's a keeper means it's a legal character to be in a word, where legal
 * is currently in ASCII [a-z,A-Z] and possibly [0-9]
 * 
 * The @,[,`, and { bracket the a-zA-Z in ASCII order
 * The / and : bracket 0-9
 * ********************************************************************* */
int IsAkeeper (
   char* instr, 
   int opt_digits
)
{
  return (     ((strcmp("@",instr) < 0) && (strcmp("[",instr) > 0))
           ||  ((strcmp("`",instr) < 0) && (strcmp("{",instr) > 0))
           || (((strcmp("/",instr) < 0) && (strcmp(":",instr) > 0)) && (opt_digits == 1))
         );
}



/* ************************************************************************
 * Lower case the word.
 * This operates directly on the string passed to it.
 * It only knows ascii at this point.
 * ********************************************************************* */
void lc (char *a_word )
{
  int  index ;
  index = 0 ;

  while (a_word[index] != '\0' )
  {
    if ( (a_word[index] < 91) && (a_word[index] > 64) )
    {
      a_word[index] = a_word[index] + 32 ;
    }
    index++ ;
  }
}

